@@ -39,7 +39,7 @@ class SelfAttention(torch.nn.Module):
     def __init__(self, hidden_size, num_attention_heads,
                  attention_dropout_prob, output_dropout_prob,
                  init_method, layer_id, hidden_size_per_attention_head=None, output_layer_init_method=None, bias=True,
-                 hooks={}, transformer_pointer=None):
+                 hooks={}, transformer_pointer=None, params_dtype=torch.float, skip_init=False, device=torch.device('cpu')):
         super(SelfAttention, self).__init__()
         # Set output layer initialization if not provided.
         if output_layer_init_method is None:
@@ -65,8 +65,11 @@ class SelfAttention(torch.nn.Module):
             gather_output=False,
             init_method=init_method,
             bias=bias,
+            params_dtype=params_dtype,
             module=self,
-            name="query_key_value"
+            name="query_key_value",
+            skip_init=skip_init,
+            device=device
         )
         self.attention_dropout = torch.nn.Dropout(attention_dropout_prob)
 
@@ -76,8 +79,11 @@ class SelfAttention(torch.nn.Module):
             input_is_parallel=True,
             init_method=output_layer_init_method,
             bias=bias,
+            params_dtype=params_dtype,
             module=self,
-            name="dense"
+            name="dense",
+            skip_init=skip_init,
+            device=device
         )
         self.output_dropout = torch.nn.Dropout(output_dropout_prob)
         
@@ -105,7 +111,7 @@ class CrossAttention(torch.nn.Module):
     """Parallel cross-attention layer for Transformer"""
 
     def __init__(self, hidden_size, num_attention_heads, attention_dropout_prob, output_dropout_prob, init_method,
-                 layer_id, hidden_size_per_attention_head=None, output_layer_init_method=None, bias=True, hooks={},transformer_pointer=None):
+                 layer_id, hidden_size_per_attention_head=None, output_layer_init_method=None, bias=True, hooks={},transformer_pointer=None, params_dtype=torch.float, skip_init=False, device=torch.device('cpu')):
         super().__init__()
         # Set output layer initialization if not provided.
         if output_layer_init_method is None:
@@ -125,11 +131,12 @@ class CrossAttention(torch.nn.Module):
         # Strided linear layer.
         self.query = ColumnParallelLinear(hidden_size, self.inner_hidden_size,
                                           gather_output=False,
-                                          init_method=init_method, bias=bias, module=self, name="query")
+                                          init_method=init_method, bias=bias, params_dtype=params_dtype, module=self, name="query", skip_init=skip_init, device=device)
         self.key_value = ColumnParallelLinear(hidden_size, 2 * self.inner_hidden_size,
                                               stride=2,
                                               gather_output=False,
-                                              init_method=init_method, bias=bias, module=self, name="key_value")
+                                              init_method=init_method, bias=bias, params_dtype=params_dtype, module=self, name="key_value",
+                                              skip_init=skip_init, device=device)
         # Dropout. Note that for a single iteration, this layer will generate
         # different outputs on different number of parallel partitions but
         # on average it should not be partition dependent.
@@ -140,7 +147,8 @@ class CrossAttention(torch.nn.Module):
             self.inner_hidden_size,
             hidden_size,
             input_is_parallel=True,
-            init_method=output_layer_init_method, bias=bias, module=self, name="dense")
+            init_method=output_layer_init_method, bias=bias, params_dtype=params_dtype, module=self, name="dense",skip_init=skip_init,
+            device=device)
         self.output_dropout = torch.nn.Dropout(output_dropout_prob)
 
         object.__setattr__(self, 'transformer', transformer_pointer)
@@ -166,7 +174,7 @@ class CrossAttention(torch.nn.Module):
 
 class MLP(torch.nn.Module):
     def __init__(self, hidden_size, output_dropout_prob, init_method, inner_hidden_size=None,
-                 output_layer_init_method=None, layer_id=None, hooks={}, bias=True, activation_func=gelu, transformer_pointer=None):
+                 output_layer_init_method=None, layer_id=None, hooks={}, bias=True, activation_func=gelu, transformer_pointer=None, params_dtype=torch.float, skip_init=False, device=torch.device('cpu')):
         super(MLP, self).__init__()
         self.layer_id = layer_id
         self.activation_func = activation_func
@@ -185,8 +193,11 @@ class MLP(torch.nn.Module):
             gather_output=False,
             init_method=init_method,
             bias=bias,
+            params_dtype=params_dtype,
             module=self,
-            name="dense_h_to_4h"
+            name="dense_h_to_4h",
+            skip_init=skip_init,
+            device=device
         )
         # Project back to h.
         self.dense_4h_to_h = RowParallelLinear(
@@ -195,8 +206,11 @@ class MLP(torch.nn.Module):
             input_is_parallel=True,
             init_method=output_layer_init_method,
             bias=bias,
+            params_dtype=params_dtype,
             module=self,
-            name="dense_4h_to_h"
+            name="dense_4h_to_h",
+            skip_init=skip_init,
+            device=device
         )
         self.dropout = torch.nn.Dropout(output_dropout_prob)
         object.__setattr__(self, 'transformer', transformer_pointer)
@@ -233,7 +247,10 @@ class BaseTransformerLayer(torch.nn.Module):
             use_bias=True,
             activation_func=gelu,
             hooks={},
-            transformer_pointer=None
+            transformer_pointer=None,
+            params_dtype=torch.float,
+            skip_init=False,
+            device=torch.device('cpu')
     ):
         super(BaseTransformerLayer, self).__init__()
         # Set output layer initialization if not provided.
@@ -261,7 +278,10 @@ class BaseTransformerLayer(torch.nn.Module):
             output_layer_init_method=output_layer_init_method,
             bias=use_bias,
             hooks=hooks,
-            transformer_pointer=transformer_pointer
+            transformer_pointer=transformer_pointer,
+            params_dtype=params_dtype,
+            skip_init=skip_init,
+            device=device
         )
 
         # Layernorm on the input data.
@@ -283,7 +303,8 @@ class BaseTransformerLayer(torch.nn.Module):
                 output_layer_init_method=output_layer_init_method,
                 bias=use_bias,
                 hooks=hooks,
-                transformer_pointer=transformer_pointer
+                transformer_pointer=transformer_pointer,
+                params_dtype=params_dtype
             )
             self.post_cross_attention_layernorm = layernorm(hidden_size, eps=layernorm_epsilon)
 
@@ -298,7 +319,10 @@ class BaseTransformerLayer(torch.nn.Module):
             layer_id=layer_id,
             activation_func=activation_func,
             hooks=hooks,
-            transformer_pointer=transformer_pointer
+            transformer_pointer=transformer_pointer,
+            params_dtype=params_dtype,
+            skip_init=skip_init,
+            device=device
         )
 
     def forward(self, hidden_states, mask, *args, **kw_args):
@@ -329,7 +353,10 @@ class BaseTransformer(torch.nn.Module):
                  layernorm=LayerNorm,
                  init_method=None,
                  use_final_layernorm=True,
-                 hooks={}
+                 hooks={},
+                 params_dtype=torch.float,
+                 skip_init=False,
+                 device=torch.device('cpu')
                  ):
         super(BaseTransformer, self).__init__()
 
@@ -347,7 +374,8 @@ class BaseTransformer(torch.nn.Module):
         self.embedding_dropout = torch.nn.Dropout(embedding_dropout_prob)
 
         self.word_embeddings = VocabParallelEmbedding(
-            vocab_size, hidden_size, init_method=unscaled_init_method(0.02))
+            num_embeddings=vocab_size, embedding_dim=hidden_size, 
+            params_dtype=params_dtype, skip_init=skip_init, device=device)
 
         self.position_embeddings = torch.nn.Embedding(max_sequence_length, hidden_size)
         torch.nn.init.normal_(self.position_embeddings.weight, mean=0.0, std=init_method_std)
@@ -379,6 +407,9 @@ class BaseTransformer(torch.nn.Module):
                 activation_func=activation_func,
                 hooks=self.hooks,
                 transformer_pointer=self,
+                params_dtype=params_dtype,
+                skip_init=skip_init,
+                device=device
             )
 
         self.layers = torch.nn.ModuleList(
@@ -394,6 +425,7 @@ class BaseTransformer(torch.nn.Module):
         # sanity check
         assert len(input_ids.shape) == 2
         batch_size, query_length = input_ids.shape
+
         if attention_mask is None:
             attention_mask = torch.ones(1, 1, device=input_ids.device).type_as(
                 next(self.parameters())
@@ -473,7 +505,7 @@ class BaseTransformer(torch.nn.Module):
                         output_cross_layer[k] = len(flat_outputs) - 1
                     # --------------------
 
-                    return x_, output_per_layers_part, output_cross_layer, flat_outputs
+                    return (x_, output_per_layers_part, output_cross_layer, *flat_outputs)
                 return custom_forward
 
             # prevent to lose requires_grad in checkpointing.
@@ -495,7 +527,7 @@ class BaseTransformer(torch.nn.Module):
                     flat_inputs.append(v)
                     cross_layer_index[k] = len(flat_inputs) - 1
                 # --------------------
-                hidden_states, output_per_layers_part, output_cross_layer, flat_outputs = \
+                hidden_states, output_per_layers_part, output_cross_layer, *flat_outputs = \
                     checkpoint(custom(l, l + chunk_length, kw_args_index, cross_layer_index), *args, *flat_inputs)
                 
                 # recover output_per_layers_part, output_cross_layer
@@ -516,8 +548,10 @@ class BaseTransformer(torch.nn.Module):
                 output_this_layer_obj, output_cross_layer_obj = {}, {}
 
                 if 'layer_forward' in self.hooks: # customized layer_forward
-                    layer_ret = self.hooks['layer_forward'](*args, layer_id=torch.tensor(i),
+                    layer_ret = self.hooks['layer_forward'](*args,
+                        layer_id=torch.tensor(i),
                         **kw_args,
+                        position_ids=position_ids,
                         **output_cross_layer,
                         output_this_layer=output_this_layer_obj, output_cross_layer=output_cross_layer_obj
                     )
